<?php

/* -----------------------------------------------------------------------------------------
  VamShop - http://vamshop.com
  -----------------------------------------------------------------------------------------
  Copyright (c) 2014 VamSoft Ltd.
  License - http://vamshop.com/license.html
  --------------------------------------------------------------------------------------- */

class OfdfermaController extends ModuleOfdfermaAppController {

    public $uses = array('ModuleOfdferma', 'ModuleOfdfermaSetting', 'OrderStatusDescription', 'Language', 'Order', 'ContentProduct');
    
    const API_URL   = 'https://ferma.ofd.ru/api/kkt/cloud';
    const AUTH_URL  = 'https://ferma.ofd.ru/api/Authorization/CreateAuthToken';
    const CHECK_URL = 'https://check.ofd.ru/rec/';
    
    private $debug = false;
    
    /**
     * @var array
     */
    protected $settings;
    
    public function __construct($request = false, $response = null)
    {
		if (false === $request){
			$request = new CakeRequest(null, false);
		}

        parent::__construct($request, $response);
        
        foreach ($this->getSettings() as $moduleOfdfermaSetting) {
            $setting = $moduleOfdfermaSetting['ModuleOfdfermaSetting'];
            $this->settings[$setting['code']] = $setting['value'];
        }
    }
    
    public function setDebug($val) {
        $this->debug = $val;
    }
    
    /**
     * @return array
     */
    public function getSettings()
    {      
        $settings = $this->ModuleOfdfermaSetting->find('all', array('order' => array('position ASC')));

        foreach ($settings as $key => $value) {

            if ($settings[$key]['ModuleOfdfermaSetting']['type'] === 'select') {
                $settings[$key]['ModuleOfdfermaSetting']['options'] = $this -> getSettingSelect($value['ModuleOfdfermaSetting']['code']);
            }
        }

        return $settings;
    }
    
    private function getSettingSelect($field){
        
        $res = array();
        
        SWITCH ($field){
            case 'osn':
            case 'nds':
                /*App::import('Vendor', 'Ofdferma', array('file' => 'ofdferma'.DS.'ofdFermaApi.php'));
                $ofdFermaApi = new ofdFermaApi();
                $res = $ofdFermaApi->getData($field);*/
                
                //App::import('Controller', 'ModuleOfdferma.AppController');
                //$ofdFermaAppController = new ModuleOfdfermaAppController();
                $res = $this->getData($field);
            break;
        
            case 'status':
                $res['0'] = __d('module_ofdferma', 'none');
                
                $language = $this->Language->find('first', ['conditions' => ['active' => 1]]);
                $statuses = $this->OrderStatusDescription->find('all', [
                    'conditions' => [
                        'language_id' => $language['Language']['id']
                    ]
                ]);

                foreach ($statuses as $status) {
                    $res[$status['OrderStatusDescription']['order_status_id']] = $status['OrderStatusDescription']['name'];
                }
            break;
        }
        
        return $res;
    }
    
    public function __d($str){
        return __d('module_ofdferma', $str);
    }
    
    public function getData($field){
        
        $res = array();
        
        SWITCH ($field){
            case 'osn':
                $res = array(
                    'Common'                => __d('module_ofdferma', 'Common'),
                    'SimpleIn'              => __d('module_ofdferma', 'SimpleIn'),
                    'SimpleInOut'           => __d('module_ofdferma', 'SimpleInOut'),
                    'Unified'               => __d('module_ofdferma', 'Unified'),
                    'UnifiedAgricultural'   => __d('module_ofdferma', 'UnifiedAgricultural'),
                    'Patent'                => __d('module_ofdferma', 'Patent')
                );
            break;
        
            case 'nds':
                $res = array(
                    'Vat0'                  => __d('module_ofdferma', 'Vat0'),
                    'Vat10'                 => __d('module_ofdferma', '10%'),
                    'Vat18'                 => __d('module_ofdferma', '18%'),
                    'Vat20'                 => __d('module_ofdferma', '20%'),
                    'CalculatedVat10110'    => __d('module_ofdferma', '10/110'),
                    'CalculatedVat18118'    => __d('module_ofdferma', '18/118'),
                    'CalculatedVat20120'    => __d('module_ofdferma', '20/120')
                );
            break;
        }
        
        return $res;
    }
    
    private function check_settings() {
        $ofd_login      = $this->settings['login'];
        $ofd_pass       = $this->settings['password'];
        $ofd_osn        = $this->settings['osn'];
        $ofd_inn        = $this->settings['inn'];
        $ofd_nds        = $this->settings['nds'];

        return (
            $ofd_login &&
            $ofd_pass &&
            $ofd_osn &&
            $ofd_inn &&
            $ofd_nds
        );
    }
    
    public function getTextType($type) {
        $res = '';

        switch ($type) {
            case 'Income':
                $res = '';
                break;

            case 'IncomeReturn':
                $res = '';
                break;
        }

        return $res;
    }
    
    private function getErrorByCode($code_id, $data) {
        $msg = '';

        SWITCH ($code_id) {
            case 1019:
                $msg = "  '{$data['Request']['InvoiceId']}'    ";
                break;
        }

        return $msg;
    }
    
    private function formatFloat( $int, $count = 2){
        return	round( (float)$int , $count);
    }
    
    private function customErrorHandler($errno, $errstr, $errfile, $errline, array $errcontext) {
        $this->log_msg($errstr);

        if (0 === error_reporting()) {
            return false;
        }
        throw new Exception($errstr, 0, $errno, $errfile, $errline);
    }
    
    private function get_receipt($order_id, $type){
        
        $conditions = array('order_id' => $order_id, 'type' => $type);
        return $this->ModuleOfdferma->find('first', ['conditions' => $conditions]);
    }
    
    public function getCheckLink($row){
        $check_link = self::CHECK_URL . "{$row['inn']}/{$row['RNM']}/{$row['FN']}/".ltrim($row['FDN'],0)."/".$row['FPD'];
        
        return $check_link;
    }
    
    public function get_product_vat($tovar_id, $default){
        $data = $this->ContentProduct->find('first', ['conditions' => ['ContentProduct.content_id' => intval($tovar_id)]]);
        return isset($data['ContentProduct']['ofdferma_nds']) && $data['ContentProduct']['ofdferma_nds'] ? $data['ContentProduct']['ofdferma_nds'] : $default;
    }
    
    public function log_msg($mess, $status = false, $params = array()) {
        if ( ! $status && $this->debug && $mess) {
            $txt = $mess;
            
            CakeLog::warning($txt);

            $user_mail = $this->settings['email'];
            if ($user_mail){  
                
                App::import('Controller', 'Component.ConfigurationBaseComponent');
                $this->ConfigurationBase = new ConfigurationBaseComponent();
                
                /*App::import('Controller', 'Component.EmailComponent');
                $this->EmailComponent = new EmailComponent();*/
                
                
                $config = $this->ConfigurationBase->load_configuration();
                
                App::import('Vendor', 'PHPMailer', array('file' => 'phpmailer'.DS.'class.phpmailer.php'));
                $mail = new PHPMailer;

                $mail->CharSet = 'utf-8';
                $mail->IsHTML(true);
                
                
		$mail->From = $config['SEND_CONTACT_US_EMAIL'];
		$mail->FromName = $config['SEND_CONTACT_US_EMAIL'];
		$mail->AddReplyTo($user_mail);
		$mail->AddAddress($config['SEND_CONTACT_US_EMAIL']);
		$mail->Subject = 'OFD Ferma - ';

		// Email Body
		$mail->Body = date('d.m.Y H:i:s') .' : '. $txt;
		
		// Sending mail
		$mail->send();
            }
        }
        
        return array_merge(array(
            'status'    => $status,
            'mess'      => $this -> json_fix_cyr($mess)
        ), (array)$params);
    } 
    
    public function set_status_order($order_id, $status_id){
        
        $res = array('status' => false);
        //         
        if ($this->settings['status'] == $status_id){
            $res = $this->Check($order_id, 'Income');
        }
        
        return $res;
    }
    
    /**
     *  Win 1251  UTF8
     * @param string $in_text
     * @return string
     */
    static function win_utf8($in_text) {
        if (function_exists('iconv')) {
            $output = iconv("windows-1251", "utf-8", $in_text);
        } else {

            $output = null;
            $other[1025] = "";
            $other[1105] = "";
            $other[1028] = "";
            $other[1108] = "";
            $other[1030] = "I";
            $other[1110] = "i";
            $other[1031] = "";
            $other[1111] = "";

            for ($i = 0; $i < strlen($in_text); $i++) {
                if (ord($in_text{$i}) > 191) {
                    $output.="&#" . (ord($in_text{$i}) + 848) . ";";
                } else {
                    if (array_search($in_text{$i}, $other) === false) {
                        $output.=$in_text{$i};
                    } else {
                        $output.="&#" . array_search($in_text{$i}, $other) . ";";
                    }
                }
            }
        }

        return $output;
    }

    /**
     *  utf8  win1251
     * @param string $s 
     * @return string
     */
    static function utf8_win1251($s) {
        $s = strtr($s, array("\xD0\xB0" => "", "\xD0\x90" => "", "\xD0\xB1" => "", "\xD0\x91" => "", "\xD0\xB2" => "", "\xD0\x92" => "", "\xD0\xB3" => "", "\xD0\x93" => "", "\xD0\xB4" => "", "\xD0\x94" => "", "\xD0\xB5" => "", "\xD0\x95" => "", "\xD1\x91" => "", "\xD0\x81" => "", "\xD0\xB6" => "", "\xD0\x96" => "", "\xD0\xB7" => "", "\xD0\x97" => "", "\xD0\xB8" => "", "\xD0\x98" => "", "\xD0\xB9" => "", "\xD0\x99" => "", "\xD0\xBA" => "", "\xD0\x9A" => "", "\xD0\xBB" => "", "\xD0\x9B" => "", "\xD0\xBC" => "", "\xD0\x9C" => "", "\xD0\xBD" => "", "\xD0\x9D" => "", "\xD0\xBE" => "", "\xD0\x9E" => "", "\xD0\xBF" => "", "\xD0\x9F" => "", "\xD1\x80" => "", "\xD0\xA0" => "", "\xD1\x81" => "", "\xD0\xA1" => "", "\xD1\x82" => "", "\xD0\xA2" => "", "\xD1\x83" => "", "\xD0\xA3" => "", "\xD1\x84" => "", "\xD0\xA4" => "", "\xD1\x85" => "", "\xD0\xA5" => "", "\xD1\x86" => "", "\xD0\xA6" => "", "\xD1\x87" => "", "\xD0\xA7" => "", "\xD1\x88" => "", "\xD0\xA8" => "", "\xD1\x89" => "", "\xD0\xA9" => "", "\xD1\x8A" => "", "\xD0\xAA" => "", "\xD1\x8B" => "", "\xD0\xAB" => "", "\xD1\x8C" => "", "\xD0\xAC" => "", "\xD1\x8D" => "", "\xD0\xAD" => "", "\xD1\x8E" => "", "\xD0\xAE" => "", "\xD1\x8F" => "", "\xD0\xAF" => ""));
        return $s;
    }
    
    function json_fix_cyr($var) {
        if (is_array($var)) {
            $new = array();
            foreach ($var as $k => $v) {
                $new[$this->json_fix_cyr($k)] = $this->json_fix_cyr($v);
            }
            $var = $new;
        } elseif (is_string($var)) {
            $var = $this->win_utf8($var);
        }
        return $var;
    }

    /**
     *    UTF-8  Windows-1251
     * @param array $var  
     * @return array
     */
    function json_fix_utf($var) {
        if (is_array($var)) {
            $new = array();
            foreach ($var as $k => $v) {
                $new[$this->json_fix_utf($k)] = $this->json_fix_utf($v);
            }
            $var = $new;
        } elseif (is_string($var)) {
            $var = $this->utf8_win1251($var);
        }
        return $var;
    }
    
    // 
    private function get_token(){
        
        if ( ! $this->check_settings()) {
            return $this->log_msg(' ,      : , ,   , ,  ');
        };
        
        if ($this->settings['token'] && ($this->settings['token_exp_date'] > (time() - 10))) {
            return $this->log_msg('', true);
        }
        
        $ofd_login      = $this->settings['login'];
        $ofd_pass       = $this->settings['password'];
        
        $ofd_auth_url   = self::AUTH_URL;

        $response = $this -> execute_request($ofd_auth_url, array(
            "Login"     => $ofd_login,
            "Password"  => $ofd_pass,
        ));
        
        if (isset($response['result'])){
            $result = $response['result'];
            
            // 
            $this->settings['token'] = $result['Data']['AuthToken'];
            $this->settings['token_exp_date'] = $result['Data']['ExpirationDateUtc'];
            
            foreach(array('token','token_exp_date') as $field){
                $currentConfig = $this->ModuleOfdfermaSetting->find('first', ['conditions' => ['code' => $field]]);
                $currentConfig['ModuleOfdfermaSetting']['value'] = $this->settings[$field];
                $this->ModuleOfdfermaSetting->save($currentConfig['ModuleOfdfermaSetting']);
            }
            
            return $this->log_msg('', true);
        }
        
        return $response;
    }
    
    private function execute_request($url, $data = null) {  
        
        $data = $this->json_fix_cyr($data);
        
        $options = array(
            "ssl" => array(
                "verify_peer" => false,
                "verify_peer_name" => false,
            ),
            'http' => array(
                'timeout' => 10,
                'ignore_errors' => true,
                'content' => json_encode($data),
                'header' => "Content-type: application/json\r\n" .
                "Accept: application/json" . "\r\n",
                "Content-Length: " . strlen(json_encode($data)) . "\r\n",
                'method' => 'POST',
            )
        );
        
        $context = stream_context_create($options);
        set_error_handler(array($this, 'customErrorHandler'));

        try {
            $result = file_get_contents($url, false, $context);
        } catch (Exception $e) {
            return $this->log_msg($e->getMessage());
        }
        
        restore_error_handler();
        $result = json_decode($result, TRUE);
        
        if (isset($result['Status']) && ($result['Status'] == 'Success')) {
            return $this->log_msg('', true, array('result' => $result));
        } else if (isset($result['Status']) && ($result['Status'] == 'Failed')) {
            $msg = $result['Error']['Message'];
            if ($result['Error']['Code']) {
                if ($val = $this->getErrorByCode($result['Error']['Code'], $data)) {
                    $msg = $val;
                }
            }
            
            return $this->log_msg($msg, false, $result);
        } else {
            return $this->log_msg('some error', false, $result);
        }
    }
    
    /**
    *     Ferma
    * @param array $data  
    * @param string $operation  [sale|sale_refund]
    * @param bool $json      JSON 
    */
   function Check($order_id, $type = 'Income') {
       
        $res = $this->get_token();
        if ( ! $res['status']) {
            return $res;
        }
       
        $order = $this->Order->read(null,$order_id);
        if ( ! $order){
            return $this->log_msg(" {$order_id}  !");
        }     
        
        
        $temptype = $this -> getTextType($type);
        
        //   
        if ( ! $order['OrderProduct']) {
            return $this->log_msg("  #{$order_id}  ");
        }
        
        if ($this -> get_receipt($order_id, $type)){
            return $this->log_msg(" {$temptype}   {$order_id}    ");
        }
        
        $res = $this->get_token();
        if ( ! $res['status']) {
            return $res;
        }
        
        //   
        $prefix     = 'o';
        
        $ofd_osn    = $this->settings['osn'];
        $ofd_inn    = $this->settings['inn'];
        $ofd_nds    = $this->settings['nds'];
        
        //
        $ofd_collapse       = $this->settings['collapse'];
        $ofd_collapse_name  = $this->settings['collapse_name'];
        
        $customer_phone = str_replace(array('(', ')', '-', ' '), '', $order['Order']['phone']);
        $customer_email = $order['Order']['email'];     

        // 
        $data = array();
        $data['Request']['Inn'] = $ofd_inn;
        $data['Request']['Type'] = $type;
        $data['Request']['InvoiceId'] = $prefix . $order_id . '-' . $type;
        $data['Request']['LocalDate'] = date('Y-m-d\TH:i:s');
        $data['Request']['CustomerReceipt'] = array(
            'TaxationSystem'    => $ofd_osn,
            'Email'             => $customer_email,
            'Phone'             => $customer_phone,
            'Items'             => array(),
        );
           
        //
        if (is_array($order['OrderProduct'])) {
            foreach ($order['OrderProduct'] as $product){
                $vat = $this->get_product_vat($product['content_id'], $ofd_nds);
                
                array_push($data['Request']['CustomerReceipt']['Items'], array(
                        'Label'     => $product['name'],
                        'Price'     => $this->formatFloat( $product['price'] ),
                        'Quantity'  => $this->formatFloat( $product['quantity'], 3),
                        'Amount'    => $this->formatFloat( $product['price'] * $product['quantity'] ),
                        'Vat'       => $vat,
                    )
                );
            }
        }

        
        // 
        if (!empty($order['Order']['shipping'])) {
            array_push($data['Request']['CustomerReceipt']['Items'], array(
                    'Label'     => '',
                    'Price'     => $this->formatFloat( $order['Order']['shipping'] ),
                    'Quantity'  => $this->formatFloat( 1, 3),
                    'Amount'    => $this->formatFloat( $order['Order']['shipping'] ),
                    'Vat'       => $ofd_nds,
                )
            );
        }
        
        //  
        if ($ofd_collapse) {
            $pos_name = $ofd_collapse_name ? $ofd_collapse_name : 'Undefined';

            $sum = 0;
            foreach ($data['Request']['CustomerReceipt']['Items'] as $item) {
                $sum += $item['Amount'];
            }
            $data['Request']['CustomerReceipt']['Items'] = array(array(
                    'Label'     => $pos_name,
                    'Price'     => $sum,
                    'Quantity'  => 1,
                    'Amount'    => $sum,
                    'Vat'       => $ofd_nds,
            ));
        }
        
        
        //   
        $api_url = self::API_URL . "/receipt?AuthToken=" . $this->settings['token'];
        $response = $this -> execute_request($api_url, $data);
        
        if (isset($response['result'])){
            $result = $response['result'];
            
            //  
            $this->ModuleOfdferma->save(array(
                'id'        => $result['Data']['ReceiptId'],
                'type'      => $data['Request']['Type'],
                'inn'       => $data['Request']['Inn'],
                'order_id'  => $order_id,
                'total'     => $order['Order']['total'],
                'created_at'=> gmdate('Y-m-d H:i:s')
            ));
                 
            return $this->log_msg(" {$temptype}   {$order_id} : {$result['Data']['ReceiptId']}", true);
        }else{
            return $response;
        }
    }
   
    //  
    public function check_update($id){
        
        $res = $this->get_token();
        if ( ! $res['status']) {
            return $res;
        }

        $results = $this->ModuleOfdferma->find('all', ['conditions' => array('id' => $id, "NOT" => array( "status" => array('FAILED')))]);
        
        $this -> setDebug(1);
        foreach ($results as $resultItem) {
            $result = $resultItem['ModuleOfdferma'];
            
            $data = array();
            $data['Request']['ReceiptId'] = $result['id'];
            
            //   
            $api_url = self::API_URL . "/status?AuthToken=" . $this->settings['token'];
            $response = $this -> execute_request($api_url, $data);

            if (isset($response['result'])){
                
                //  
                $this->ModuleOfdferma->save(array(
                    'id'                => $result['id'],
                    'status'            => $response['result']['Data']['StatusName'],
                    'status_message'    => $response['result']['Data']['StatusMessage'],
                    'FN'                => $response['result']['Data']['Device']['FN'],
                    'RNM'               => $response['result']['Data']['Device']['RNM'],
                    'FDN'               => $response['result']['Data']['Device']['FDN'],
                    'FPD'               => $response['result']['Data']['Device']['FPD'],
                    'updated_at'        => gmdate('Y-m-d H:i:s')
                ));
            }
        }
        $this -> setDebug(0);
    }
    
    
    // 
    public function get_ofdferma(){}
}

?>